
/**
 ******************************************************************************
 * @file    main.c
 * @author  psavr@gmx.ch, dyk1990@arcor.de
 * @date    09-September-2013
 * @brief   Demo-Application for STM32F4-Discovery Board
 *          including RTOS CoOS V1.1.4 (2011.04.20)
 *          including USART module for printf() on USART2, USART3 or USART6
 *          including command shell for terminal on USART6 at 115200-8-N-1
 *          including generic I2C functions
 *          including Webserverdemo with UART CGI Handler, ADC Reading, switching LEDS
 * @detail  To open Webserver, connect your STM32-Discovery Board via BaseBoard DM-STF4BB to
 * 			an ethernet switch and connect a client computer to the same switch using two patch cables.
 * 			Configure your client with a static IP adress like 192.168.0.100, subnet 255.255.255.0
 * 			and standard gateway 192.168.2.1. Then you should be able to call
 * 			the webserver tiping in your browsers adress field http://192.168.0.10
 * 			(which is the adress of	the STM32 Webserver, configured in main.h)
 ******************************************************************************
 * @attention
 *
 * This Example is maintained under the PsAvr "BEERWARE-LICENSE"
 * Feel free to use it, to modify it and to distribute it. If you like
 * it and we meet us once in a pub or a bar, feel free to spend us a beer!
 *
 * I would appreciate it very much, if you would return any corrections
 * and/or improvements back to me and/or to the CooCox community.
 *
 *****************************************************************************
 *
 * The webserver application was added by a student working at Mixed Mode GmbH, Munich, Germany
 * as vacation developer. The software was taken from mikrocontroller.net and modified
 * respecting the licence conditions. In order to contribute this project and animate
 * other developers to improve it, this code is given back to the community.
 * Because this is a non profit project, there's absolutely no warranty on functionality.
 ******************************************************************************
 */

/*---------------------------- Include ---------------------------------------*/

#include "main.h"
#include "usart.h"
#include "i2c.h"
#include "shell.h"
#include "delay.h"

/*---------------------------- Symbol Define -------------------------------*/
#define STACK_SIZE_TASKA 128              /*!< Define "taskA" task size */
#define STACK_SIZE_TASKB 128              /*!< Define "taskB" task size */
#define STACK_SIZE_TASKC 128              /*!< Define "taskC" task size */
#define STACK_SIZE_TASKD 128              /*!< Define "taskD" task size */
#define STACK_SIZE_TASKE 2048              /*!< Define "taskD" task size */
#define STACK_SIZE_SHELL 1024             /*!< Define "shell" task size */


/*---------------------------- Variable Define -----------------------------*/

OS_STK taskA_stk[STACK_SIZE_TASKA]; /*!< Define "taskA" task stack */
OS_STK taskB_stk[STACK_SIZE_TASKB]; /*!< Define "taskB" task stack */
OS_STK taskC_stk[STACK_SIZE_TASKC]; /*!< Define "taskC" task stack */
OS_STK taskD_stk[STACK_SIZE_TASKD]; /*!< Define "taskD" task stack */
OS_STK taskE_stk[STACK_SIZE_TASKE]; /*!< Define "taskE" task stack */
OS_STK shell_stk[STACK_SIZE_SHELL]; /*!< Define "shell" task stack */

/*---------------------------- Function Prototypes -------------------------*/

void LED_Init(void);
void taskA(void* pdata);
void taskB(void* pdata);
void taskC(void* pdata);
void taskD(void* pdata);
void taskE(void* pdata);

/*---------------------------- Function Code -------------------------------*/
/**
 *******************************************************************************
 * @brief       "Terminal" task code
 * @param[in]   None
 * @param[out]  None
 * @retval      None
 * @par Description
 * @details    This task is the terminal-demon for the stdio
 *******************************************************************************
*/

#ifndef _SHELL_H_
void Shell(void* pdata)
{
  static char msg[128];
  CoTickDelay(10);
  printf("CoOS Terminal-Task: started\n>");
  fflush(0);
  while (1)
  {
    CoTickDelay(1);
    if(UART_GetLine(msg))
    {
      printf(">Unknown Command: '%s'\n", msg);
      printf(">");
      fflush(0);
    }
  }
}
#endif

/**
 *******************************************************************************
 * @brief       "taskA" task code
 * @param[in]   None
 * @param[out]  None
 * @retval      None
 * @par Description
 * @details    This task use to crate mutex and flags,print message "taskA running".
 *             Indicate "taskA" had been executed.
 *******************************************************************************
 */
void taskA(void* pdata)
{
  unsigned int a = 0;
  printf("CoOS taskA: started\n");
  while (1)
  {
    if (a & 1)
    {
      GPIO_SetBits(GPIOD, GPIO_Pin_12);
    } else
    {
      GPIO_ResetBits(GPIOD, GPIO_Pin_12);
    }
    a++;
    CoTickDelay(25); //25 x 10ms = 250ms
  }
}

/**
 *******************************************************************************
 * @brief       "taskB" task code
 * @param[in]   None
 * @param[out]  None
 * @retval      None
 * @par Description
 * @details    This task use to print message "taskB running". Indicate "taskB"
 *             had been executed.
 *******************************************************************************
 */
void taskB(void* pdata)
{
  unsigned int b = 0;
  printf("CoOS taskB: started\n");
  while (1)
  {
    if (b & 1)
    {
      GPIO_SetBits(GPIOD, GPIO_Pin_13);
    } else
    {
      GPIO_ResetBits(GPIOD, GPIO_Pin_13);
    }
    b++;

    CoTickDelay(50); //50 x 10ms = 500ms
  }
}

/**
 *******************************************************************************
 * @brief       "taskC" task code
 * @param[in]   None
 * @param[out]  None
 * @retval      None
 * @par Description
 * @details    This task use to print message "taskC running". Indicate "taskC"
 *             had been executed.
 *******************************************************************************
 */
void taskC(void* pdata)
{
  unsigned int c = 0;
  printf("CoOS taskC: started\n");
  while (1)
  {
    if (c & 1)
    {
      GPIO_SetBits(GPIOD, GPIO_Pin_14);
    } else
    {
      GPIO_ResetBits(GPIOD, GPIO_Pin_14);
    }
    c++;
    CoTickDelay(100); //100 x 10ms = 1000ms
  }
}

/**
 *******************************************************************************
 * @brief       "taskD" task code
 * @param[in]   None
 * @param[out]  None
 * @retval      None
 * @par Description
 * @details    This task use to print message "taskD running". Indicate "taskD"
 *             had been executed.
 *******************************************************************************
 */
void taskD(void* pdata)
{
  unsigned int d = 0;
  printf("CoOS taskD: started\n");
  while (1)
  {
	if (d & 1)
    {
      GPIO_ResetBits(GPIOD, GPIO_Pin_15);
    } else
    {
      GPIO_SetBits(GPIOD, GPIO_Pin_15);
    }
    d++;
    CoTickDelay(50); //50 x 10ms = 500ms
  }
}

void taskE(void* pdata)
{
  printf("CoOS taskEthernet: started\n");
  while (1)
  {
	  /* check if any packet received */
	      	    if (ETH_CheckFrameReceived())
	      	    {
	      	      /* process received ethernet packet */
	      	      LwIP_Pkt_Handle();
	      	    }
	      	    /* handle periodic timers for LwIP */
	      	    LwIP_Periodic_Handle((uint32_t)OSTickCnt);
    CoTickDelay(1); //1 x 10ms = 500ms
  }
}

void LED_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  /* GPIOD Periph clock enable */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
  /* Configure PD12, PD13, PD14 and PD15 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = (GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);
}

/*---------------------------- Main function -------------------------------*/

int main(void)
{
  UART_Init();
  I2C_BusInit();
  LED_Init();

  printf("\n---------------------------------------\n");
  printf("STM32F4-Discovery Board is boooting!\n");
  printf("CoOS RTOS V-1.1.4; Demo for STM32F4\n");
  printf("---------------------------------------\n");
  delay_ms(20);  // some wait-time to clear UART-Buffer

  /* configure ethernet (GPIOs, clocks, MAC, DMA) */
  ETH_BSP_Config();

  /* Initilaize the LwIP stack */
  LwIP_Init();

  /* Http webserver Init */
  httpd_init();


  /*!< Initialize CooCox CoOS          */
  CoInitOS();

  /*!< Create tasks	*/
  printf("CoOS RTOS: Creating tasks\n");
//  CoCreateTask(taskA, 0, 10, &taskA_stk[STACK_SIZE_TASKA-1], STACK_SIZE_TASKA);
//  CoCreateTask(taskB, 0, 20, &taskB_stk[STACK_SIZE_TASKB-1], STACK_SIZE_TASKB);
  CoCreateTask(Shell, 0, 25, &shell_stk[STACK_SIZE_SHELL-1], STACK_SIZE_SHELL);
//  CoCreateTask(taskC, 0, 30, &taskC_stk[STACK_SIZE_TASKC-1], STACK_SIZE_TASKC);
//  CoCreateTask(taskD, 0, 40, &taskD_stk[STACK_SIZE_TASKD-1], STACK_SIZE_TASKD);
  CoCreateTask(taskE, 0, 50, &taskE_stk[STACK_SIZE_TASKE-1], STACK_SIZE_TASKE);

  printf("CoOS RTOS: Starting scheduler\n");
  CoStartOS(); /*!< Start multitask	           */

  printf("CoOS RTOS: Scheduler stopped\n");

	while (1)
  {
	  NVIC_SystemReset();    // nope
  }
}

